/**
* Manage Contacts
* It will be your responsibility to fine tune this template, add validations, try/catch blocks, logging, etc.
*/
component{
	
	// DI Virtual Entity Service
	property name="ormService" inject="entityService:Contact";
	
	// HTTP Method Security
	this.allowedMethods = {
		index = "GET", new = "GET", edit = "GET", delete = "POST,DELETE", save = "POST,PUT"
	};
	
	/**
	* preHandler()
	*/
	function preHandler(event){
		event.paramValue("format","html");
	}
		
	/**
	* Listing
	*/
	function index(event,rc,prc) cache="true" cacheTimeout="10"{
		// Get all Contacts
		rc.Contacts = ormService.getAll();
		
		// RESTful Switch
		switch(rc.format){
			// xml,json,jsont,wddx are by default.  Add your own or remove
			case "xml" : case "json" : case "jsont" : case "wddx" :{
				event.renderData(data=rc.Contacts,type=rc.format);
				break;
			}
			// HTML
			default:{
				event.setView("Contacts/index");
			}
		}
	}	
	
	/**
	* New Form
	*/
	function new(event,rc,prc){
		
		// get new Contact
		rc.Contact = ormService.new();
		
		event.setView("Contacts/new");
	}	

	/**
	* Edit Form
	*/
	function edit(event,rc,prc){
		
		// get persisted Contact
		rc.Contact = ormService.get( rc.contactID );
		
		event.setView("Contacts/edit");
	}	
	
	/**
	* View Contact mostly used for RESTful services only.
	*/
	function show(event,rc,prc){
		
		// Get requested entity by id
		rc.Contact = ormService.get( rc.contactID );
		
		switch(rc.format){
			// xml,json,jsont,wddx are by default.  Add your own or remove
			case "xml" : case "json" : case "jsont" : case "wddx" :{
				event.renderData(data=rc.Contact,type=rc.format);
				break;
			}
			// HTML
			default:{
				setNextEvent('Contacts');
			}
		}
	}

	/**
	* Save and Update
	*/
	function save(event,rc,prc){
		
		// get Contact to persist or update and populate it with incoming form
		rc.Contact = populateModel(model=ormService.get( rc.contactID ),exclude="contactID");
		
		// Do your validations here
		
		// Save it
		ormService.save( rc.Contact );
		
		// RESTful Handler
		switch(rc.format){
			// xml,json,jsont,wddx are by default.  Add your own or remove
			case "xml" : case "json" : case "jsont" : case "wddx" :{
				event.renderData(data=rc.Contact,type=rc.format,location="/Contacts/show/#rc.Contact.getcontactID()#");
				break;
			}
			// HTML
			default:{
				// Show a nice messagebox
				getplugin("MessageBox").info("Contact Created!");
				// Redirect to listing
				setNextEvent('Contacts');
			}
		}
	}	

	/**
	* Delete
	*/
	function delete(event,rc,prc){
		
		// Delete record by ID
		var removed = ormService.deleteById( rc.contactID );
		
		// RESTful Handler
		switch(rc.format){
			// xml,json,jsont,wddx are by default.  Add your own or remove
			case "xml" : case "json" : case "jsont" : case "wddx" :{
				var restData = {deleted=removed};
				event.renderData(data=restData,type=rc.format);
				break;
			}
			// HTML
			default:{
				// Show a nice messagebox
				getplugin("MessageBox").warn("Contact Poofed!");
				// Redirect to listing
				setNextEvent('Contacts');
			}
		}
	}	
	
}